@@ -45,7 +45,7 @@ impl_Bonobo_PropertyBag_getKeys (PortableServer_Servant  servant,
 	char            *path;
 	Bonobo_KeyList  *retval;
 	GSList          *slist, *sl;
-	GError          *err;
+	GError          *err = NULL;
 	int              length;
 	int              n;
 
@@ -93,7 +93,7 @@ impl_Bonobo_PropertyBag_getType (PortableServer_Servant  servant,
 	BonoboConfigBag *cb = GET_BAG_FROM_SERVANT (servant);
 	char            *path;
 	GConfValue      *value;
-	GError          *err;
+	GError          *err = NULL;
 
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -131,6 +131,33 @@ impl_Bonobo_PropertyBag_getType (PortableServer_Servant  servant,
 	return CORBA_OBJECT_NIL;
 }
 
+static BonoboArg*
+bonobo_arg_new_from_gconf_value (GConfValue *value)
+{
+        if (value == NULL)
+                return bonobo_arg_new (BONOBO_ARG_NULL);
+        
+	switch (value->type) {
+	case GCONF_VALUE_STRING :
+		return bonobo_arg_new_from (BONOBO_ARG_STRING,
+					    gconf_value_get_string (value));
+	case GCONF_VALUE_INT : {
+                long v = gconf_value_get_int (value);
+		return bonobo_arg_new_from (BONOBO_ARG_LONG, &v);
+        }
+	case GCONF_VALUE_FLOAT : {
+                double v = gconf_value_get_float (value);
+		return bonobo_arg_new_from (BONOBO_ARG_DOUBLE, &v);
+        }
+	case GCONF_VALUE_BOOL : {
+                gboolean v = gconf_value_get_bool (value);
+		return bonobo_arg_new_from (BONOBO_ARG_BOOLEAN, &v);
+        }
+	default :
+		return bonobo_arg_new (BONOBO_ARG_NULL);
+	}
+}
+
 static CORBA_any *
 impl_Bonobo_PropertyBag_getValue (PortableServer_Servant  servant,
 				  const CORBA_char       *key,
@@ -139,7 +166,7 @@ impl_Bonobo_PropertyBag_getValue (PortableServer_Servant  servant,
 	BonoboConfigBag *cb = GET_BAG_FROM_SERVANT (servant);
 	char            *path;
 	GConfValue      *value;
-	GError          *err;
+	GError          *err = NULL;
  
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -156,21 +183,10 @@ impl_Bonobo_PropertyBag_getValue (PortableServer_Servant  servant,
 		return CORBA_OBJECT_NIL;
 	}
 
-	switch (value->type) {
-	case GCONF_VALUE_STRING :
-		return bonobo_arg_new_from (BONOBO_ARG_STRING,
-					    gconf_value_get_string (value));
-	case GCONF_VALUE_INT :
-		return bonobo_arg_new_from (BONOBO_ARG_LONG, &value->d.int_data);
-	case GCONF_VALUE_FLOAT :
-		return bonobo_arg_new_from (BONOBO_ARG_DOUBLE, &value->d.float_data);
-	case GCONF_VALUE_BOOL :
-		return bonobo_arg_new_from (BONOBO_ARG_BOOLEAN, &value->d.bool_data);
-	default :
-		return bonobo_arg_new (BONOBO_ARG_NULL);
-	}
-
-	return CORBA_OBJECT_NIL;
+        /* FIXME The original code here returned BonoboArg*
+         * as a CORBA_any*, is that OK?
+         */
+        return bonobo_arg_new_from_gconf_value (value);
 }
 
 static void 
@@ -181,7 +197,7 @@ impl_Bonobo_PropertyBag_setValue (PortableServer_Servant  servant,
 {
 	BonoboConfigBag *cb = GET_BAG_FROM_SERVANT (servant);
 	char            *path;
-	GError          *err;
+	GError          *err = NULL;
 	
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -232,7 +248,7 @@ impl_Bonobo_PropertyBag_getValues (PortableServer_Servant servant,
 	char *path;
 	Bonobo_PropertySet *retval;
 	GSList *slist, *sl;
-	GError *err;
+	GError *err = NULL;
 	int length;
 	int n;
 
@@ -262,29 +278,13 @@ impl_Bonobo_PropertyBag_getValues (PortableServer_Servant servant,
 	for (sl = slist, n = 0; n < length; sl = sl->next, n++) {
 		GConfEntry *entry = (GConfEntry *) sl->data;
 		BonoboArg *arg;
+                GConfValue *value;
 
 		retval->_buffer[n].name = CORBA_string_dup (gconf_entry_get_key (entry));
-		switch (entry->value->type) {
-		case GCONF_VALUE_STRING :
-			arg = bonobo_arg_new_from (BONOBO_ARG_STRING,
-						   gconf_value_get_string (entry->value));
-			break;
-		case GCONF_VALUE_INT :
-			arg = bonobo_arg_new_from (BONOBO_ARG_LONG,
-						   &entry->value->d.int_data);
-			break;
-		case GCONF_VALUE_FLOAT :
-			arg = bonobo_arg_new_from (BONOBO_ARG_DOUBLE,
-						   &entry->value->d.float_data);
-			break;
-		case GCONF_VALUE_BOOL :
-			arg = bonobo_arg_new_from (BONOBO_ARG_BOOLEAN,
-						   &entry->value->d.bool_data);
-			break;
-		default :
-			arg = bonobo_arg_new (BONOBO_ARG_NULL);
-		}
+                value = gconf_entry_get_value (entry);
 
+                arg = bonobo_arg_new_from_gconf_value (value);
+                
 		retval->_buffer[n].value = *arg;
 	}
 
@@ -318,7 +318,7 @@ impl_Bonobo_PropertyBag_getDefault (PortableServer_Servant  servant,
 	BonoboConfigBag *cb = GET_BAG_FROM_SERVANT (servant);
 	char            *path;
 	GConfValue      *value;
-	GError          *err;
+	GError          *err = NULL;
 
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -335,21 +335,7 @@ impl_Bonobo_PropertyBag_getDefault (PortableServer_Servant  servant,
 		return CORBA_OBJECT_NIL;
 	}
 
-	switch (value->type) {
-	case GCONF_VALUE_STRING :
-		return bonobo_arg_new_from (BONOBO_ARG_STRING,
-					    gconf_value_get_string (value));
-	case GCONF_VALUE_INT :
-		return bonobo_arg_new_from (BONOBO_ARG_LONG, &value->d.int_data);
-	case GCONF_VALUE_FLOAT :
-		return bonobo_arg_new_from (BONOBO_ARG_DOUBLE, &value->d.float_data);
-	case GCONF_VALUE_BOOL :
-		return bonobo_arg_new_from (BONOBO_ARG_BOOLEAN, &value->d.bool_data);
-	default :
-		/* FIXME */
-	}
-
-	return CORBA_OBJECT_NIL;
+        return bonobo_arg_new_from_gconf_value (value);
 }
 
 static CORBA_char *
@@ -361,7 +347,7 @@ impl_Bonobo_PropertyBag_getDocTitle (PortableServer_Servant  servant,
 	char            *path;
 	CORBA_char      *retval;
 	GConfSchema     *schema;
-	GError          *err;
+	GError          *err = NULL;
 
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -393,7 +379,7 @@ impl_Bonobo_PropertyBag_getDoc (PortableServer_Servant  servant,
 	char            *path;
 	CORBA_char      *retval;
 	GConfSchema     *schema;
-	GError          *err;
+	GError          *err = NULL;
 
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -426,7 +412,7 @@ impl_Bonobo_PropertyBag_getFlags (PortableServer_Servant  servant,
 	char                 *path;
 	Bonobo_PropertyFlags  retval = 0;
 	GConfEntry           *entry;
-	GError               *err;
+	GError               *err = NULL;
 
 	if (strchr (key, '/')) {
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
